﻿Public Class TypingForm
   Private lastButton As Button ' reference to last Button pressed
   Private keyDictionary As New Dictionary(Of String, String)

   ' handles Form's KeyDown Event
   Private Sub outputTextBox_KeyDown(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.KeyEventArgs) _
      Handles outputTextBox.KeyDown

      If Char.IsControl(Convert.ToChar(e.KeyCode)) OrElse _
         IsFunctionOrArrowKey(e.KeyCode) Then

         ' locate the Button representing the pressed key with LINQ
         Dim pressed = _
            From currentControl In Me.Controls _
            Where TypeOf currentControl Is Button AndAlso _
               keyDictionary.ContainsKey( _
                  e.KeyCode.ToString()) AndAlso _
               keyDictionary(e.KeyCode.ToString()) = _
                  CType(currentControl, Button).Text _
            Select currentControl

         If pressed.Count > 0 Then
            ChangeColor(CType(pressed.First, Button))
         End If
      End If
   End Sub ' OutputTextBox_KeyDown

   ' handles Form KeyPress Event
   Private Sub outputTextBox_KeyPress(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.KeyPressEventArgs) _
      Handles outputTextBox.KeyPress

      ' locate the Button matching the pressed key with LINQ
      Dim pressed = _
         From currentControl In Me.Controls _
         Where TypeOf currentControl Is Button AndAlso _
            e.KeyChar.ToString().ToUpper() = _
               CType(currentControl, Button).Text _
         Select currentControl

      If e.KeyChar = " " Then ' if user pressed spacebar
         ChangeColor(spaceButton)
      Else
         If pressed.Count > 0 Then
            ChangeColor(CType(pressed.First, Button))
         End If
      End If
   End Sub ' outputTextBox_KeyPress

   ' handles the TextBox's KeyUp event
   Private Sub outputTextBox_KeyUp(ByVal sender As Object, _
      ByVal e As System.Windows.Forms.KeyEventArgs) _
      Handles outputTextBox.KeyUp

      ResetColor()
   End Sub ' outputTextBox_KeyUp

   ' highlight Button passed as argument
   Private Sub ChangeColor(ByVal buttonPassed As Button)
      ResetColor()
      buttonPassed.BackColor = Color.Yellow
      lastButton = buttonPassed ' save Button to reset color later
   End Sub ' ChangeColor

   ' changes lastButton's color if it refers to a Button
   Private Sub ResetColor()
      If lastButton IsNot Nothing Then
         lastButton.BackColor = SystemColors.Control
      End If
   End Sub ' ResetColor

   ' configure keyDictionary for use with control keys;
   ' key is the key code's string value; value is the Button label
   Private Sub TypingForm_Load(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles MyBase.Load

      ' add function keys
      For i As Integer = 1 To 12
         keyDictionary.Add("F" & i, "F" & i)
      Next

      ' add other control keys
      keyDictionary.Add("Back", "Backspace")
      keyDictionary.Add("Return", "Enter")
      keyDictionary.Add("ControlKey", "Ctrl")
      keyDictionary.Add("Menu", "Alt")
      keyDictionary.Add("Capital", "Caps Lock")
      keyDictionary.Add("ShiftKey", "Shift")
      keyDictionary.Add("Tab", "Tab")
      keyDictionary.Add("Up", "^")
      keyDictionary.Add("Down", "v")
      keyDictionary.Add("Left", "<")
      keyDictionary.Add("Right", ">")
   End Sub ' TypingForm_Load

   ' determine whether pressed key is a function or arrow key
   Function IsFunctionOrArrowKey(ByVal code As Keys) As Boolean
      Dim result As Boolean

      Select Case code
         Case Keys.F1 To Keys.F12
            result = True
         Case Keys.Up, Keys.Down, Keys.Left, Keys.Right
            result = True
         Case Else
            result = False ' not a match
      End Select

      Return result
   End Function ' isFunctionOrArrowKey

   ' handles Font menu item's Click event
   Private Sub fontMenuItem_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles fontMenuItem.Click

      Dim dialog As New FontDialog
      Dim result As DialogResult ' stores Button clicked

      ' show dialog and get result
      result = dialog.ShowDialog()

      ' do nothing if user clicked dialog's Cancel Button
      If result = System.Windows.Forms.DialogResult.Cancel Then
         Return
      End If

      ' assign new font value to TextBox
      outputTextBox.Font = dialog.Font
   End Sub ' fontMenuItem_Click

   ' handles Color menu item's Click event
   Private Sub colorMenuItem_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles colorMenuItem.Click

      Dim dialog As New ColorDialog
      Dim result As DialogResult ' stores Button clicked

      dialog.FullOpen = True ' show all colors
      result = dialog.ShowDialog()

      ' do nothing if user clicked dialog's Cancel Button
      If result = System.Windows.Forms.DialogResult.Cancel Then
         Return
      End If

      ' assign new color value to TextBox
      outputTextBox.ForeColor = dialog.Color
   End Sub ' colorMenuItem_Click

   ' handles Clear TextBox menu item's Click Event
   Private Sub clearMenuItem_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles clearMenuItem.Click

      outputTextBox.Clear() ' clear TextBox
   End Sub ' clearMenuItem_Click

   ' handles Invert Colors menu item's Click Event
   Private Sub invertMenuItem_Click(ByVal sender As System.Object, _
      ByVal e As System.EventArgs) Handles invertMenuItem.Click

      Dim temporaryColor As Color ' temporary Color value

      temporaryColor = outputTextBox.BackColor
      outputTextBox.BackColor = outputTextBox.ForeColor
      outputTextBox.ForeColor = temporaryColor
   End Sub ' invertMenuItem_Click
End Class ' Typing

' **************************************************************************
' * (C) Copyright 1992-2009 by Deitel & Associates, Inc. and               *
' * Pearson Education, Inc. All Rights Reserved.                           *
' *                                                                        *
' * DISCLAIMER: The authors and publisher of this book have used their     *
' * best efforts in preparing the book. These efforts include the          *
' * development, research, and testing of the theories and programs        *
' * to determine their effectiveness. The authors and publisher make       *
' * no warranty of any kind, expressed or implied, with regard to these    *
' * programs or to the documentation contained in these books. The authors *
' * and publisher shall not be liable in any event for incidental or       *
' * consequential damages in connection with, or arising out of, the       *
' * furnishing, performance, or use of these programs.                     *
' **************************************************************************
